"use strict";

exports.__esModule = true;
exports.default = void 0;

var _compile = require("./compile");

var _styleResolver = _interopRequireDefault(require("./styleResolver"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) 2016-present, Nicolas Gallagher.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/**
 * A simple (and dangerous) CSS system.
 * The order of CSS rule insertion is not guaranteed.
 * Avoiding combining 2 or more classes that modify the same property.
 */
var css = {
  /**
   * const classes = css.create({ base: {}, extra: {} })
   */
  create: function create(rules) {
    var result = {};
    Object.keys(rules).forEach(function (name) {
      var style = rules[name];
      var compiled = (0, _compile.classic)(style, name);
      Object.values(compiled).forEach(function (_ref) {
        var identifier = _ref.identifier,
            rules = _ref.rules;
        rules.forEach(function (rule) {
          _styleResolver.default.sheet.insert(rule, _constants.STYLE_GROUPS.classic);
        });
        result[name] = identifier;
      });
    });
    return result;
  },

  /**
   * css.combine(classes.base, classes.extra)
   */
  combine: function combine() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return args.reduce(function (className, value) {
      if (value) {
        className += className.length > 0 ? ' ' + value : value;
      }

      return className;
    }, '');
  }
};
var _default = css;
exports.default = _default;
module.exports = exports.default;